import xml.etree.ElementTree as ET
import game
import os
import fileutils
from project import Project

logs = []


def log(msg):
    logs.append(msg)


def validate():
    units = game.get_units_list()
    index = 0
    while os.path.isfile(fileutils.root_dir + '/Resources/ini/maps/map%d.xml' % index):
        root = ET.parse(fileutils.root_dir + '/Resources/ini/maps/map%d.xml' % index).getroot()
        waves = root.find('waves')
        routes = root.find('routes')
        units_on_map = {}

        def validate_route(routeindex):
            try:
                int(routeindex)
            except:
                log('Route with index [{}] not is digit. map{}.xml'.format(routeindex, index))
                return False
            for route in routes:
                if route.attrib['name'] == routeindex:
                    return True
            log('Route with index [{}] not found in map{}.xml'.format(routeindex, index))
            return False

        def validate_routesubtype(rst):
            if rst not in ['main', 'left', 'right', 'random']:
                log('Unknow value of routesubtype [{}] in map{}.xml'.format(rst, index))

        for wave in waves:
            if 'defaultname' in wave.attrib:
                unit = wave.attrib['defaultname']
                if unit not in units:
                    log('Invalid name of unit [{}] in map{}.xml'.format(unit, index))
                units_on_map[unit] = 1
            if 'defaultrouteindex' in wave.attrib:
                validate_route(wave.attrib['defaultrouteindex'])
            if 'defaultroutesubtype' in wave.attrib:
                validate_routesubtype(wave.attrib['defaultroutesubtype'])
            for unitxml in wave:
                unit = unitxml.attrib['name'] if 'name' in unitxml.attrib else ''
                units_on_map[unit] = 1
                if unit and unit not in units:
                    log('Invalid name of unit [{}] in map{}.xml'.format(unit, index))
                if 'routeindex' in unitxml.attrib:
                    validate_route(unitxml.attrib['routeindex'])
                if 'routesubtype' in unitxml.attrib:
                    validate_routesubtype(unitxml.attrib['routesubtype'])
        if 'max_creeps_on_level' in Project.instance.validate:
            max_count = Project.instance.validate['max_creeps_on_level']
        else:
            max_count = 8
        if len(units_on_map) > max_count:
            log('Many creeps in map{}.xml ({}>{})'.format(index, len(units_on_map), max_count))
        index += 1



import build_tasks as task

ConfigurationDebug = 0
ConfigurationRelease = 1
ConfigurationPreRelease = 2


class PlatformBuild(object):

    def __init__(self):
        super(PlatformBuild, self).__init__()

    def set_version(self, packagename, app_version, build_version):
        return False

    def build(self, configuration):
        return False

    def upload(self):
        pass

    def upload_inapps(self):
        pass


class PlatformWindow(PlatformBuild):

    def __init__(self):
        super(PlatformWindow, self).__init__()

    def set_version(self, packagename, app_version, build_version):
        return True

    def build(self, configuration):
        strs = ['GameDebug', 'GameRelease', 'GameRelease']
        return task.windows_build(strs[configuration])


class PlatformAndroid(PlatformBuild):

    def __init__(self):
        super(PlatformAndroid, self).__init__()

    def build(self, configuration):
        gradleTasks = ['assembleDebug', 'assembleRelease', 'assembleRelease']
        return task.android_build(gradleTasks[configuration])

    def set_version(self, packagename, app_version, build_version):
        task.android_change_version(packagename, app_version, build_version)

    def upload(self):
        task.android_upload_apk()


class PlatformIos(PlatformBuild):

    def __init__(self):
        super(PlatformIos, self).__init__()

    def build(self, configuration):
        if configuration == ConfigurationDebug:
            print 'ios build not supported Debug configuration. Use one [release, pre-release]'
            return False

        scheme = self.arg_parser.scheme
        task.ios_build_archive(scheme)
        task.ios_export_archive()

        return True

    def set_version(self, packagename, app_version, build_version):
        task.ios_change_version(packagename, app_version, build_version)

    def upload(self):
        task.ios_upload_archive()

    def upload_inapps(self):
        task.ios_upload_inapps()


def create_builder(arg_parser):
    platform = arg_parser.platform
    platforms = {'windows': PlatformWindow, 'android': PlatformAndroid, 'ios': PlatformIos}
    builder = platforms[platform]()
    builder.arg_parser = arg_parser
    return builder


def run(package_name, app_version, arg_parser):
    configurations = {'debug': ConfigurationDebug, 'release': ConfigurationRelease, 'pre-release': ConfigurationPreRelease, }
    configuration = configurations[arg_parser.configuration]

    builder = create_builder(arg_parser)
    builder.set_version(package_name, app_version, arg_parser.build_version)
    builder.build(configuration)
    if configuration == ConfigurationRelease:
        builder.upload()


def upload(arg_parser):
    builder = create_builder(arg_parser)
    builder.upload()


def upload_inapps(arg_parser):
    builder = create_builder(arg_parser)
    builder.upload_inapps()



import fileutils
import validator_xml

import validate_maps
import validator_localization
import validate_sounds


def print_log(validator, caption):
    print caption
    validator.validate()
    for i, log in enumerate(sorted(validator.logs)):
        print '\t', str(i + 1) + '\t: ', log
    if not validator.logs:
        print '\tSuccess'


def run():
    result = True
    fileutils.ignore_folders.extend(['steam', 'cocoscenes'])
    fileutils.inspectResources()

    # validate xmls
    for file in fileutils.xmls:
        validator_xml.validate(file)
    logs = sorted(validator_xml.logs)
    for i, log in enumerate(logs):
        print '\t', str(i + 1) + '\t: ', log
    print '{} errors in {} xmls'.format(len(logs), len(fileutils.xmls))
    print 'Unused atlases:\n\t', '\n\t'.join(fileutils.unused) if len(fileutils.unused) else ' not founded'

    print_log(validate_maps, 'Validate maps:')
    print_log(validator_localization, 'Validate localization:')
    print_log(validate_sounds, 'Validate sounds:')

    result = result and len(validator_xml.logs) == 0
    result = result and len(validate_maps.logs) == 0
    result = result and len(validator_localization.logs) == 0
    if not result:
        print 'Validation finished with Failed result'
        exit(-1)
    print 'Validation finished.'


if __name__ == '__main__':
    run()



import json
import fileutils
import os


class Services:

    def __init__(self):
        self.appodeal_id = ''
        self.flurry_id = ''
        self.facebook_id = ''
        self.gsm_sender_id = ''


class Project(object):

    instance = None

    def __init__(self, arg_parser, empty=False):
        super(Project, self).__init__()
        self.with_pro_version = False
        self.services = None
        self.arg_parser = arg_parser
        if not empty:
            self._parse()

    def create(self):
        data = {}
        data['name'] = ''
        data['app_package'] = ''
        data['app_name'] = ''
        data['app_bundle_name'] = ''
        data['app_version'] = ''
        data['google_spreedsheet_inapps_id'] = ''
        data['google_api_secret_path'] = ''
        data['apple_auth_user'] = ''
        data['apple_auth_password'] = ''
        data['apple_id'] = ''
        data['apple_team_id'] = ''

        # services
        data['services'] = {}
        data['services']['ios'] = {}
        data['services']['android'] = {}
        data['services']['ios']['flurry_id'] = ''
        data['services']['ios']['appodeal_id'] = ''
        data['services']['ios']['facebook_id'] = ''
        data['services']['ios']['gsm_sender_id'] = ''
        data['services']['android']['flurry_id'] = ''
        data['services']['android']['appodeal_id'] = ''
        data['services']['android']['facebook_id'] = ''
        data['services']['android']['gsm_sender_id'] = ''

        path = fileutils.root_dir + '/project.json'
        open(path, 'w').write(json.dumps(data, sort_keys=True, indent=4))
        fileutils.createDir(fileutils.root_dir + '/store')
        fileutils.createDir(fileutils.root_dir + '/store/inapps.itmsp')
        fileutils.write(fileutils.root_dir + '/store/inapps.itmsp/machine-local-data.xml', '<root/>')
        fileutils.write(fileutils.root_dir + '/store/inapps.itmsp/metadata.xml', '<root/>')
        fileutils.write(fileutils.root_dir + '/store/android_inapps.csv',
                        'Product ID,Published State,Purchase Type,Auto Translate,Locale; Title; Description,Auto Fill Prices,Price,Pricing Template ID')

    def _parse(self):
        path = fileutils.root_dir + '/project.json'
        if not os.path.isfile(path):
            print 'Cannot find project file [project.json]'
            exit(-1)
        data_file = open(path)
        data = json.load(data_file)
        self.project_name = data['name'] if 'name' in data else None
        self.package = data['app_package']
        self.name = data['app_name']
        self.app_bundle_name = data['app_bundle_name'] if 'app_bundle_name' in data else self.name
        self.version = data['app_version']
        self.gg_inapps = data['google_spreedsheet_inapps_id']
        self.gg_secret_file = fileutils.root_dir + '/' + data['google_api_secret_path']
        self.apple_auth_user = data['apple_auth_user']
        self.apple_auth_password = data['apple_auth_password']
        self.apple_id = data['apple_id']
        self.apple_team_id = data['apple_team_id']
        self.validate = data['validator_properties'] if 'validator_properties' in data else {}
        if self.project_name is None:
            k = self.package.rfind('.')
            self.project_name = self.package[k + 1:]

        if 'services' in data:
            def read(platform):
                js = data['services'][platform]
                if 'lite' in js and 'pro' in js:
                    self.with_pro_version = True
                    self.services[platform]['lite'] = Services()
                    self.services[platform]['lite'].flurry_id = js['lite']['flurry_id']
                    self.services[platform]['lite'].appodeal_id = js['lite']['appodeal_id']
                    self.services[platform]['lite'].facebook_id = js['lite']['facebook_id']
                    self.services[platform]['lite'].gsm_sender_id = js['lite']['gsm_sender_id']
                    self.services[platform]['pro'] = Services()
                    self.services[platform]['pro'].flurry_id = js['pro']['flurry_id']
                    self.services[platform]['pro'].appodeal_id = js['pro']['appodeal_id']
                    self.services[platform]['pro'].facebook_id = js['pro']['facebook_id']
                    self.services[platform]['pro'].gsm_sender_id = js['pro']['gsm_sender_id']
                else:
                    self.with_pro_version = False
                    self.services[platform] = Services()
                    self.services[platform].flurry_id = js['flurry_id']
                    self.services[platform].appodeal_id = js['appodeal_id']
                    self.services[platform].facebook_id = js['facebook_id']
                    self.services[platform].gsm_sender_id = js['gsm_sender_id']

            self.services = {}
            self.services['ios'] = {}
            self.services['android'] = {}
            read('ios')
            read('android')
        if self.with_pro_version and self.arg_parser.app_kind != 'lite':
            self.package += '.' + self.arg_parser.app_kind
